@@ -14,7 +14,7 @@ FireTasks)
 to do next after a job completes
 """
 
-from collections import defaultdict
+from collections import defaultdict, OrderedDict
 
 import datetime
 from fireworks.core.fworker import FWorker
@@ -177,8 +177,7 @@ class FireWork(FWSerializable):
         self.fw_id = fw_id
         self.launches = launches if launches else []
         self.archived_launches = archived_launches if archived_launches else []
-        self.created_on = created_on if created_on else datetime.datetime \
-            .utcnow()
+        self.created_on = created_on if created_on else datetime.datetime.utcnow()
 
         self.state = state
 
@@ -483,7 +482,8 @@ class Workflow(FWSerializable):
         def from_dict(cls, m_dict):
             return Workflow.Links(m_dict)
 
-    def __init__(self, fireworks, links_dict=None, name=None, metadata=None):
+    def __init__(self, fireworks, links_dict=None, name=None, metadata=None, created_on=None,
+                 updated_on=None):
         """
         :param fireworks: ([FireWork]) - all FireWorks in this workflow
         :param links_dict: (dict) links between the FWs as (parent_id):[(
@@ -515,6 +515,8 @@ class Workflow(FWSerializable):
             raise ValueError("Specified links don't match given FW")
 
         self.metadata = metadata if metadata else {}
+        self.created_on = created_on if created_on else datetime.datetime.utcnow()
+        self.updated_on = updated_on if updated_on else datetime.datetime.utcnow()
 
     @property
     def fws(self):
@@ -532,9 +534,10 @@ class Workflow(FWSerializable):
             m_state = 'DEFUSED'
         elif any([s == 'FIZZLED' for s in states]):
             m_state = 'FIZZLED'
-        elif any([s == 'COMPLETED' for s in states]) or any(
-                [s == 'RUNNING' for s in states]):
+        elif any([s == 'COMPLETED' for s in states]) or any([s == 'RUNNING' for s in states]):
             m_state = 'RUNNING'
+        elif any([s == 'RESERVED' for s in states]):
+            m_state = 'RESERVED'
 
         return m_state
 
@@ -625,11 +628,11 @@ class Workflow(FWSerializable):
 
             self.id_fw[new_fw.fw_id] = new_fw  # add new_fw to id_fw
 
-            if new_fw.fw_id in leaf_ids and detour:
-                self.links[new_fw.fw_id] = self.links[
-                    fw_id]  # add children of current FW to new FW
-            elif new_fw.fw_id in leaf_ids:
-                self.links[new_fw.fw_id] = []
+            if new_fw.fw_id in leaf_ids:
+                if detour:
+                    self.links[new_fw.fw_id] = list(self.links[fw_id])  # add children of current FW to new FW
+                else:
+                    self.links[new_fw.fw_id] = []
             else:
                 self.links[new_fw.fw_id] = wf.links[new_fw.fw_id]
             updated_ids.append(new_fw.fw_id)
@@ -703,6 +706,8 @@ class Workflow(FWSerializable):
                 updated_ids = updated_ids.union(
                     self.refresh(child_id, updated_ids))
 
+        self.updated_on = datetime.datetime.utcnow()
+
         return updated_ids
 
     @property
@@ -756,35 +761,45 @@ class Workflow(FWSerializable):
         return {'fws': [f.to_dict() for f in self.id_fw.itervalues()],
                 'links': self.links.to_dict(),
                 'name': self.name,
-                'metadata': self.metadata}
+                'metadata': self.metadata, 'updated_on': self.updated_on}
 
     def to_db_dict(self):
         m_dict = self.links.to_db_dict()
         m_dict['metadata'] = self.metadata
         m_dict['state'] = self.state
         m_dict['name'] = self.name
+        m_dict['created_on'] = self.created_on
+        m_dict['updated_on'] = self.updated_on
         return m_dict
 
     def to_display_dict(self):
         m_dict = self.to_db_dict()
-        m_dict['states'] = dict([(self.id_fw[x].name + '--' + str(x), self.id_fw[x].state) for x in
-                            m_dict['nodes']])
-        m_dict['nodes'] = [self.id_fw[x].name for x in m_dict['nodes']]
-        m_dict['links'] = dict(
+        nodes = sorted(m_dict['nodes'])
+        m_dict['name--id'] = self.name + '--' + str(nodes[0])
+        m_dict['launch_dirs'] = OrderedDict(
+            [(self.id_fw[x].name + '--' + str(x), [l.launch_dir for l in self.id_fw[x].launches])
+             for x in nodes])
+        m_dict['states'] = OrderedDict(
+            [(self.id_fw[x].name + '--' + str(x), self.id_fw[x].state) for x in nodes])
+        m_dict['nodes'] = [self.id_fw[x].name for x in nodes]
+        m_dict['links'] = OrderedDict(
             [(self.id_fw[int(k)].name, [self.id_fw[int(v)].name for v in a]) for k, a in
              m_dict['links'].iteritems()])
-        m_dict['parent_links'] = dict(
+        m_dict['parent_links'] = OrderedDict(
             [(self.id_fw[int(k)].name, [self.id_fw[int(v)].name for v in a]) for k, a in
              m_dict['parent_links'].iteritems()])
+        m_dict['states_list'] = '-'.join([a[0:4] for a in m_dict['states'].values()])
         return m_dict
 
     @classmethod
     def from_dict(cls, m_dict):
         # accept either a Workflow dict or a FireWork dict
         if 'fws' in m_dict:
+            created_on = m_dict.get('created_on')
+            updated_on = m_dict.get('updated_on')
             return Workflow([FireWork.from_dict(f) for f in m_dict['fws']],
                             Workflow.Links.from_dict(m_dict['links']), m_dict.get('name'),
-                            m_dict['metadata'])
+                            m_dict['metadata'], created_on, updated_on)
         else:
             return Workflow.from_FireWork(FireWork.from_dict(m_dict))
 
