@@ -9,10 +9,11 @@ which specifies a QueueAdapter as well as desired properties of the submit scrip
 
 import os
 import glob
+import string
 import time
 from fireworks.core.fworker import FWorker
 from fireworks.utilities.fw_serializers import load_object
-from fireworks.utilities.fw_utilities import get_fw_logger, log_exception, create_datestamp_dir
+from fireworks.utilities.fw_utilities import get_fw_logger, log_exception, create_datestamp_dir, get_slug
 from fireworks.core.fw_config import FWConfig
 
 __author__ = 'Anubhav Jain, Michael Kocher'
@@ -58,6 +59,12 @@ def launch_rocket_to_queue(launchpad, fworker, qadapter, launcher_dir='.', reser
                 l_logger.debug('finding a FW to reserve...')
                 fw, launch_id = launchpad._reserve_fw(fworker, launcher_dir)
                 l_logger.info('reserved FW with fw_id: {}'.format(fw.fw_id))
+
+                # set job name to the FW name
+                job_name = get_slug(fw.name)
+                job_name = job_name[0:20] if len(job_name)>20 else job_name
+                qadapter.update({'job_name': job_name})  # set the job name to FW name
+
                 if '_queueadapter' in fw.spec:
                     l_logger.debug('updating queue params using FireWork spec..')
                     qadapter.update(fw.spec['_queueadapter'])
