@@ -6,6 +6,7 @@ A runnable script for managing a FireWorks database (a command-line interface to
 
 from argparse import ArgumentParser
 import os
+from pymongo import DESCENDING, ASCENDING
 from fireworks.core.fw_config import FWConfig
 from fireworks.core.launchpad import LaunchPad
 from fireworks.core.firework import Workflow
@@ -41,16 +42,25 @@ def lpad():
     adddir_parser.add_argument('wf_dir', help="path to a directory containing only FireWorks or Workflow files")
 
     get_fw_parser = subparsers.add_parser('get_fws', help='get information about FireWorks')
-    get_fw_parser.add_argument('-i', '--fw_id', help='fw_id', default=None, type=int)
-    get_fw_parser.add_argument('-n', '--name', help='name', default=None)
-    get_fw_parser.add_argument('-q', '--query', help='query (as pymongo string, enclose in single-quotes)', default=None)
-    get_fw_parser.add_argument('-d', '--display_format', help='display_format ("all","more", "less","ids")', default=None)
+    get_fw_parser.add_argument('-i', '--fw_id', help='get FW with this fw_id', default=None, type=int)
+    get_fw_parser.add_argument('-n', '--name', help='get FWs with this name', default=None)
+    get_fw_parser.add_argument('-s', '--state', help='get FWs with this state', default=None)
+    get_fw_parser.add_argument('-q', '--query', help='get FWs matching this query (as pymongo string, enclose in single-quotes)', default=None)
+    get_fw_parser.add_argument('-d', '--display_format', help='display_format ("all","more", "less","ids", "count")', default=None)
+    get_fw_parser.add_argument('-m', '--max', help='limit results', default=0, type=int)
+    get_fw_parser.add_argument('--sort', help='sort results ("created_on")', default=None)
+    get_fw_parser.add_argument('--rsort', help='reverse sort results ("created_on")', default=None)
+
 
     get_wf_parser = subparsers.add_parser('get_wfs', help='get information about Workflows')
-    get_wf_parser.add_argument('-i', '--fw_id', help='fw_id', default=None, type=int)
-    get_wf_parser.add_argument('-n', '--name', help='name', default=None)
-    get_wf_parser.add_argument('-q', '--query', help='query (as pymongo string, enclose in single-quotes)', default=None)
-    get_wf_parser.add_argument('-d', '--display_format', help='display_format ("all","more", "less","ids")', default=None)
+    get_wf_parser.add_argument('-i', '--fw_id', help='get WF with this fw_id', default=None, type=int)
+    get_wf_parser.add_argument('-n', '--name', help='get WFs with this name', default=None)
+    get_wf_parser.add_argument('-s', '--state', help='get WFs with this state', default=None)
+    get_wf_parser.add_argument('-q', '--query', help='get WFs matching this query (as pymongo string, enclose in single-quotes)', default=None)
+    get_wf_parser.add_argument('-d', '--display_format', help='display_format ("all","more", "less","ids", "count")', default=None)
+    get_wf_parser.add_argument('-m', '--max', help='limit results', default=0, type=int)
+    get_wf_parser.add_argument('--sort', help='sort results ("created_on", "updated_on")', default=None)
+    get_wf_parser.add_argument('--rsort', help='reverse sort results ("created_on", "updated_on")', default=None)
 
     rerun_fw = subparsers.add_parser('rerun_fw', help='re-run a FireWork (reset its previous launches)')
     rerun_fw.add_argument('fw_id', help='FireWork id', type=int)
@@ -136,35 +146,52 @@ def lpad():
             lp.tuneup()
 
         elif args.command == 'get_wfs':
-            if sum([bool(x) for x in [args.fw_id, args.name, args.query]]) > 1:
-                raise ValueError('Pleases specify exactly one of (fw_id, name, query)')
-            if sum([bool(x) for x in [args.fw_id, args.name, args.query]]) == 0:
+            if sum([bool(x) for x in [args.fw_id, args.name, args.state, args.query]]) > 1:
+                raise ValueError('Please specify exactly one of (fw_id, name, state, query)')
+            if sum([bool(x) for x in [args.fw_id, args.name, args.state, args.query]]) == 0:
                 args.query = '{}'
                 args.display_format = args.display_format if args.display_format else 'ids'
             else:
-                args.display_format = args.display_format if args.display_format else 'all'
+                args.display_format = args.display_format if args.display_format else 'more'
 
             if args.fw_id:
                 query = {'nodes': args.fw_id}
             elif args.name:
                 query = {'name': args.name}
+            elif args.state:
+                query = {'state': args.state}
             else:
                 query = ast.literal_eval(args.query)
 
-            ids = lp.get_wf_ids(query)
+            if args.sort:
+                sort = [(args.sort, ASCENDING)]
+            elif args.rsort:
+                sort = [(args.rsort, DESCENDING)]
+            else:
+                sort = None
+
+            ids = lp.get_wf_ids(query, sort, args.max)
             wfs = []
             if args.display_format == 'ids':
                 wfs = ids
+            elif args.display_format == 'count':
+                wfs = [len(ids)]
             else:
                 for id in ids:
                     wf = lp.get_wf_by_fw_id(id)
                     d = wf.to_display_dict()
                     if args.display_format == 'more' or args.display_format == 'less':
+                        del d['name']
                         del d['parent_links']
                         del d['nodes']
-                    if args.display_format == 'less':
                         del d['links']
-
+                    if args.display_format == 'less':
+                        del d['states']
+                        del d['metadata']
+                        del d['launch_dirs']
+                        del d['updated_on']
+                    if args.display_format == 'more' or args.display_format == 'all':
+                        del d['states_list']
                     wfs.append(d)
             if len(wfs) == 1:
                 wfs = wfs[0]
@@ -172,25 +199,36 @@ def lpad():
             print json.dumps(wfs, default=DATETIME_HANDLER, indent=4)
 
         elif args.command == 'get_fws':
-            if sum([bool(x) for x in [args.fw_id, args.name, args.query]]) > 1:
-                raise ValueError('Pleases specify exactly one of (fw_id, name, query)')
-            if sum([bool(x) for x in [args.fw_id, args.name, args.query]]) == 0:
+            if sum([bool(x) for x in [args.fw_id, args.name, args.state, args.query]]) > 1:
+                raise ValueError('Pleases specify exactly one of (fw_id, name, state, query)')
+            if sum([bool(x) for x in [args.fw_id, args.name, args.state, args.query]]) == 0:
                 args.query = '{}'
                 args.display_format = args.display_format if args.display_format else 'ids'
             else:
-                args.display_format = args.display_format if args.display_format else 'all'
+                args.display_format = args.display_format if args.display_format else 'more'
 
             if args.fw_id:
                 query = {'fw_id': args.fw_id}
             elif args.name:
                 query = {'name': args.name}
+            elif args.state:
+                query = {'state': args.state}
             else:
                 query = ast.literal_eval(args.query)
 
-            ids = lp.get_fw_ids(query)
+            if args.sort:
+                sort = [(args.sort, ASCENDING)]
+            elif args.rsort:
+                sort = [(args.rsort, DESCENDING)]
+            else:
+                sort = None
+
+            ids = lp.get_fw_ids(query, sort, args.max)
             fws = []
             if args.display_format == 'ids':
                 fws = ids
+            elif args.display_format == 'count':
+                fws = [len(ids)]
             else:
                 for id in ids:
                     fw = lp.get_fw_by_id(id)
@@ -198,10 +236,11 @@ def lpad():
                     if args.display_format == 'more' or args.display_format == 'less':
                         if 'archived_launches' in d:
                             del d['archived_launches']
+                        del d['spec']
+                    if args.display_format == 'less':
                         if 'launches' in d:
                             del d['launches']
-                    if args.display_format == 'less':
-                        del d['spec']
+
 
                     fws.append(d)
             if len(fws) == 1:
