@@ -47,6 +47,7 @@ DATETIME_HANDLER = lambda obj: obj.isoformat() if isinstance(obj, datetime.datet
 
 
 def _recursive_dict(obj):
+
     if obj is None:
         return None
 
@@ -124,7 +125,6 @@ def recursive_deserialize(func):
 
     return _decorator
 
-
 def serialize_fw(func):
     """
     a decorator to add FW serializations keys
@@ -143,16 +143,16 @@ class FWSerializable():
     """
     To create a serializable object within FireWorks, you should subclass this class and implement
     the to_dict() and from_dict() methods.
-
+    
     If you want the load_object() implicit de-serialization to work, you must also:
         - Use the @serialize_fw decorator on to_dict()
         - Override the _fw_name parameter with a unique key.
-
+    
     See documentation of load_object() for more details.
-
-    The use of @classmethod for from_dict allows you to define a superclass
+    
+    The use of @classmethod for from_dict allows you to define a superclass 
     that implements the to_dict() and from_dict() for all its subclasses.
-
+    
     For an example of serialization, see the class QueueAdapterBase.
     """
 
@@ -173,17 +173,16 @@ class FWSerializable():
     def from_dict(cls, m_dict):
         raise NotImplementedError('FWSerializable object did not implement from_dict()!')
 
-    def to_format(self, f_format='json', **kwargs):
+    def to_format(self, f_format='json', *args, **kwargs):
         """
         returns a String representation in the given format
         :param f_format: the format to output to (default json)
         """
         if f_format == 'json':
-            return json.dumps(self.to_dict(), default=DATETIME_HANDLER, **kwargs)
+            return json.dumps(self.to_dict(), *args, default=DATETIME_HANDLER, **kwargs)
         elif f_format == 'yaml':
             # start with the JSON format, and convert to YAML
-            return yaml.dump(self.to_dict(), default_flow_style=FWConfig().YAML_STYLE,
-                             allow_unicode=True)
+            return yaml.dump(self.to_dict(), default_flow_style=FWConfig().YAML_STYLE, allow_unicode=True)
         else:
             raise ValueError('Unsupported format {}'.format(f_format))
 
@@ -201,25 +200,25 @@ class FWSerializable():
         else:
             raise ValueError('Unsupported format {}'.format(f_format))
 
-    def to_file(self, filename, f_format=None, **kwargs):
+    def to_file(self, filename, f_format='AUTO_DETECT', *args, **kwargs):
         """
         Write a serialization of this object to a file
         :param filename: filename to write to
         :param f_format: serialization format, default checks the filename extension
         """
-        if f_format is None:
+        if f_format == 'AUTO_DETECT':
             f_format = filename.split('.')[-1]
         with open(filename, 'w') as f:
-            f.write(self.to_format(f_format=f_format, **kwargs))
+            f.write(self.to_format(f_format=f_format, *args, **kwargs))
 
     @classmethod
-    def from_file(cls, filename, f_format=None):
+    def from_file(cls, filename, f_format='AUTO_DETECT'):
         """
         Load a serialization of this object from a file
         :param filename: filename to read
         :param f_format: serialization format, default checks the filename extension
         """
-        if f_format is None:
+        if f_format == 'AUTO_DETECT':
             f_format = filename.split('.')[-1]
         with open(filename, 'r') as f:
             return cls.from_format(f.read(), f_format=f_format)
@@ -230,18 +229,18 @@ def load_object(obj_dict):
     """
     Creates an instantiation of a class based on a dictionary representation. We implicitly
     determine the Class through introspection along with information in the dictionary.
-
+    
     We search for a class with the _fw_name property equal to obj_dict['_fw_name']
     If the @module key is set, that module is checked first for a matching class
     to improve speed of lookup.
     Afterwards, the modules in the USER_PACKAGES global parameter are checked.
-
+    
     Refactoring class names, module names, etc. will not break object loading
     as long as:
-
+    
     i) the _fw_name property is maintained the same AND
     ii) the refactored module is kept within USER_PACKAGES
-
+    
     You can get around these limitations if you really want:
     i) If you want to change the fw_name of an object you can set the FW_NAME_UPDATES key
     ii) If you want to put a refactored module in a new place add an entry to USER_PACKAGES
@@ -262,38 +261,28 @@ def load_object(obj_dict):
 
     # failing that, look for the object within all of USER_PACKAGES
     # this will be slow, but only needed the first time
-
-    found_objects = [] # used to make sure we don't find multiple hits
     for package in FWConfig().USER_PACKAGES:
         root_module = importlib.import_module(package)
-        for loader, module_name, is_pkg in pkgutil.walk_packages(root_module.__path__,
-                                                                 package + '.'):
+        for loader, module_name, is_pkg in pkgutil.walk_packages(root_module.__path__, package + '.'):
             m_module = loader.find_module(module_name).load_module(module_name)
             m_object = _search_module_for_obj(m_module, obj_dict)
             if m_object:
-                found_objects.append((m_object, module_name))
-
-    if len(found_objects) == 1:
-        SAVED_FW_MODULES[fw_name] = found_objects[0][1]
-        return found_objects[0][0]
-    elif len(found_objects) > 0:
-        raise ValueError(
-            'load_object() found multiple objects with cls._fw_name {} -- {}'.format(fw_name,
-                                                                                     found_objects))
+                SAVED_FW_MODULES[fw_name] = module_name
+                return m_object
 
     raise ValueError('load_object() could not find a class with cls._fw_name {}'.format(fw_name))
 
 
-def load_object_from_file(filename, f_format=None):
+def load_object_from_file(filename, f_format='AUTO_DETECT'):
     """
     implicitly load an object from a file. just a friendly wrapper to load_object()
-
+    
     :param filename: the filename to load an object from
     :param f_format: the serialization format (default is auto-detect based on filename extension)
     """
 
     m_dict = {}
-    if f_format is None:
+    if f_format == 'AUTO_DETECT':
         f_format = filename.split('.')[-1]
 
     with open(filename, 'r') as f:
